-----------------------------------------------------------------
--  File        : cdc_sync.vhd
--  Version     : 1
--  Revision    : 0
--  Date        : 11.05.2018
--  Author      : Mark Harvey
--  Description : Clock Domain crossing example - XPM_CDC_GRAY
------------------------------------------------------------------

library ieee ;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library xpm;
use xpm.vcomponents.all;


entity cdc is            
  port (
    CLK50MHz      : in std_logic;                    
    CLK33MHz      : in std_logic;
    IS_FOUR       : out std_logic              
    );
end entity cdc;

architecture arc_cdc of cdc is     

   -- internal signals
   signal counter1      : unsigned(15 downto 0);
   signal counter1_slv  : std_logic_vector(15 downto 0);
   signal counter1_sync : std_logic_vector(15 downto 0);

    
begin
   
  -- 16bit up counter
  process (CLK50MHz)
  begin
    if (CLK50MHz'event and CLK50MHz = '1') then
        counter1 <= counter1 + 1;
    end if;
  end process;
  
  counter1_slv <= std_logic_vector(counter1);
  
  
  
  -- XPM_CDC_GRAY
  u0: xpm_cdc_gray
  generic map (
    -- Common module generics
    DEST_SYNC_FF          => 4,    -- integer; range: 2-10
    INIT_SYNC_FF          => 0,    -- integer; 0=disable simulation init values, 1=enable simulation init values
    REG_OUTPUT            => 0,    -- integer; 0=disable registered output, 1=enable registered output
    SIM_ASSERT_CHK        => 0,    -- integer; 0=disable simulation messages, 1=enable simulation messages
    SIM_LOSSLESS_GRAY_CHK => 0,    -- integer; 0=disable lossless check, 1=enable lossless check
    WIDTH                 => 16    -- integer; range: 2-32
  )
  port map (
    src_clk      => CLK50MHz,
    src_in_bin   => counter1_slv,
    dest_clk     => CLK33MHz,
    dest_out_bin => counter1_sync
  );
  



  -- Compare
  process (CLK33MHz)
    begin
    if (CLK33MHz'event and CLK33MHz = '1') then
      if (counter1_sync = x"0004") then
        IS_FOUR <= '1';
	   else
        IS_FOUR <= '0';
		end if;
    end if;
  end process;
        


end architecture arc_cdc;
-----------------------------------------------------------------
--  End of File: cdc_sync.vhd
-----------------------------------------------------------------
